unit GX_MessageBox;

{ TODO -oAnyone -cFeature : Allow the caller to specify an array of dialog buttons }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type

  TGxMsgBoxAdaptor = class(TObject)
  protected
    function GetMessage: string; virtual; abstract;
    function GetCaption: string; virtual;
    function GetUniqueIdentifier: string; virtual;
    procedure DoPermanentlySuppress; virtual;
    function IsPermanentlySuppressed: Boolean; virtual;
    function ShouldShow: Boolean; virtual;
  end;

  TfmGxMessageBox = class(TForm)
    btnOK: TButton;
    chkNeverShowAgain: TCheckBox;
    bvlFrame: TBevel;
    mmoMessage: TMemo;
  end;

  TGxMsgBoxAdaptorClass = class of TGxMsgBoxAdaptor;

function ShowGxMessageBox(AdaptorClass: TGxMsgBoxAdaptorClass): TModalResult;

implementation

{$R *.DFM}

uses
  Registry,
  GX_ConfigurationInfo;

function ShowGxMessageBox(AdaptorClass: TGxMsgBoxAdaptorClass): TModalResult;
var
  Adaptor: TGxMsgBoxAdaptor;
begin
  Result := mrOK;
  Adaptor := AdaptorClass.Create;
  try
    if Adaptor.IsPermanentlySuppressed then
      Exit;
    if Adaptor.ShouldShow then
    begin
      with TfmGxMessageBox.Create(nil) do
      try
        Caption := Adaptor.GetCaption;
        mmoMessage.Lines.Text := Adaptor.GetMessage;
        Result := ShowModal;
        if chkNeverShowAgain.Checked then
          Adaptor.DoPermanentlySuppress;
      finally
        Release;
      end;
    end;
  finally
    Adaptor.Free;
  end;
end;

{ TGxMsgBoxAdaptor }

function TGxMsgBoxAdaptor.GetCaption: string;
begin
  Result := 'GExperts Message';
end;

function TGxMsgBoxAdaptor.GetUniqueIdentifier: string;
begin
  Result := Self.ClassName;
end;

const
  RegistryKeyName = 'SuppressedMessages';

procedure TGxMsgBoxAdaptor.DoPermanentlySuppress;
begin
  with TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts') do
  try
    WriteBool(RegistryKeyName, Self.ClassName, True); // Do not localize
  finally
    Free;
  end;
end;

function TGxMsgBoxAdaptor.IsPermanentlySuppressed: Boolean;
begin
  with TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts') do
  try
    Result := ReadBool(RegistryKeyName, Self.ClassName, False); // Do not localize
  finally
    Free;
  end;
end;

function TGxMsgBoxAdaptor.ShouldShow: Boolean;
begin
  Result := True;
end;

end.
